# This is a test file for Config::Abstract
# It contains the same data as the file format specific
# test files

$settings = {
              'book' => {
                          'chapter1' => {
                                          'title' => 'The First Chapter, ever',
                                          'file' => 'book/chapter1.txt'
                                        },
                          'title' => 'A book of chapters',
                          'chapter2' => {
                                          'title' => 'The Next Chapter, after the First Chapter, ever',
                                          'file' => 'book/chapter2.txt'
                                        },
                          'author' => 'Me, Myself and Irene'
                        }
            };
